#
#	MoSatSAT.rb - Ruby version of SatSAT for mobile
#
#	SatSAT, a set of programs developed by JA1BML, produces satellite 
#	forcast form individual data disclosed on the JAMSAT web page. 
#
#	MoSatSAT.rb is a script written in Ruby, and performs all the 
#	functions provided by the original SatSAT, i.e., file dowonloading 
#   from the web, text re-formatting and sorting. Resulting HTML is 
#   optimized for mobile use.
#   
#	ver.0.1 JE0AWL    4 Apr. 2009
#   ver.0.2 JE0AWL    1 May  2009
#   ver.0.3 JE0AWL   10 Nov. 2009
#   ver.1.0 JE0AWL   23 Dec. 2009
#   ver.1.1 JE0AWL   27 Dec. 2009
#   ver.1.2 JE0AWL    7 May  2010
#   ver.1.3 JE0AWL   17 May  2010
#

require 'open-uri'
require 'net/http'
Net::HTTP.version_1_2

base_url = "http://www.jamsat.or.jp/pred/"  
loc = "tokyo/"
satlist = {
	"VO" => "hamsat.txt",   # VO-52
	"FO" => "fo-29.txt",    # FO-29
	"A5" => "ao-51.txt",    # AO-51
	"A2" => "ao-27.txt",    # AO-27
	"S5" => "so-50.txt",    # SO-50
	"S6" => "so-67.txt",    # SO-67
	"HO" => "xw-1.txt"      # HO-68
}
#base_dir = "C:\\MyDocuments\\Web\\sat\\"
base_dir = ".\\"
filelist = {
	"MoSatSAT_LIN"  => "(VO|FO|HL)$",
#	"MoSatSAT_FM"   => "(A5|A2|S5|S6|HF)$",
#	"MoSatSAT_MIX"  => "..$",
#	"MoSatSAT"      => "(VO|FO)$"
};
ext = ".html"
#ext = ".txt"

mmm2mm = {
	"Jan" => "01", "Feb" => "02", "Mar" => "03", "Apr" => "04", 
	"May" => "05", "Jun" => "06", "Jul" => "07", "Aug" => "08", 
	"Sep" => "09", "Oct" => "10", "Nov" => "11", "Dec" => "12"
}

data = Array.new
ho_schedule = Array.new
dates = Hash.new

date = Time.now.strftime("%y%m%d")

if satlist.member?("HO")
	dateBuf = 0
	Net::HTTP.start('www.camsat.cn', 80) {|http|
	response = http.get('/index.php?option=com_content&view=frontpage&Itemid=75&lang=en') 
	response.body.each_line {|line|

		if line =~ /(\d\d)\s*(\w+)\s*\d\d(\d\d)\<br\s*\/\>=+\<br\s*\/\>/
  			dateBuf = ($3 + mmm2mm[$2] + $1)
  			line = line.gsub($&, '')
  		end
  		if line =~ /\<p\>(\d\d)\:(\d\d)\.\.\.Turn On--(CW\/SSB|SSB\/CW|FM).+(\d\d)\:(\d\d)\.\.\.Turn Off\<\/p\>/
  			dateOn = dateBuf
  			dateOff = dateBuf
  			
  			if $1 > $4
  				dateOn = (dateBuf.to_i-1).to_s
  			end
  			
  			hourOn = sprintf("%02d", $1.to_i + 9)
  			if hourOn.to_i > 23
  				dateOn = (dateOn.to_i + 1).to_s
  				hourOn = sprintf("%02d", hourOn.to_i.modulo(24))
  			end
  			
  			hourOff = sprintf("%02d", $4.to_i + 9)
  			if hourOff.to_i > 23
  				dateOff = (dateOff.to_i + 1).to_s
  				hourOff = sprintf("%02d", hourOff.to_i.modulo(24))
  			end

			ho_schedule.push([dateOn + hourOn + $2, dateOff + hourOff + $5, $3=="FM"?"HF":"HL"])
		end
  		}
	}
end

if true
satlist.each {|sat, satfile|
	
	finp = open((base_url + loc + satfile), "r")
	p "open " + satfile
	data_buf = Array.new
	n = 0
	finp.each {|line|
		
		if line =~ /^(Sun|Mon|Tue|Wed|Thu|Fri|Sat)\s+(\d\d)(\w\w\w)(\d\d)\s+(.+)\s+\w\w\w\s+\d+/
			data_buf[0] = ($4 + mmm2mm[$3] + $2 + $1)
			
			if (not dates.value?(data_buf[0])) 
				dates[dates.size] = data_buf[0]
			end
			
			buf = $5
		elsif line =~ /^\s+(.+)\s+\w\w\w\s+\d+/
			buf = $1
		else
			next
		end
	
		if buf =~ /(\d\d):(\d\d):\d\d\s+\d\d:(\d\d):\d\d\s+(\d\d):(\d\d):\d\d\s+\d\d:(\d\d):\d\d\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\.\d.*/
			data_buf[1] = sprintf("%s%s-%s-%s(%2d) %X%X%X %2d", $1, $2, $3, $5, $6.to_i, ($7.to_i+15)/30%12, ($8.to_i+15)/30%12, ($9.to_i+15)/30%12, $10.to_i)
			data_buf[2] = sat
			
			if sat=="HO"
				aos = data_buf[0][0..5]+$1+$2
				if $1>$4
					los = (data_buf[0][0..5].to_i+1).to_s+$4+$5  # potential bug is here: increment date when end of the mounth
				else
					los = data_buf[0][0..5]+$4+$5
				end
				
				while los.to_i>ho_schedule[n][0].to_i
					n = n + 1
					if n==ho_schedule.size
						break
					end
				end

				n = n - 1
				if aos<=ho_schedule[n][1]
					p [ho_schedule[n][0], aos, los, ho_schedule[n][1], ho_schedule[n][2]]
					data_buf[2] = ho_schedule[n][2]
					data.push(data_buf.join(" "))
				end
			else
				data.push(data_buf.join(" "))
			end
		end
	
	}
	
	finp.close
}

data = data.sort

filelist.each {|filename, sats|
	fout = open(base_dir + filename + ext, "w")
	fout.puts("<html><head><title>"+filename+"</title></head><body><a name=\"top\"><br></a>") if ext==".html"
	
	date = Time.now.strftime("%y%m%d")
	dates.values.sort.each{|val|
		if val[0..5]<date
			dates.delete(dates.index(val))
			next
		end
		fout.puts("<a href=\"\#"+val[4..8]+"\">"+val[4..8]+"</a>")
	}
	fout.puts("<pre>")
	
	res = Regexp.new(sats)
	
	n = 0
	while data[n][0..8]<dates.values.sort[0]
		n = n + 1
	end
	
	dates.values.sort.each{|val|
		fout.puts("</pre><br><a name=\""+val[4..8]+"\">"+val[4..8]+"</a>   <a href=\"\#top\">top</a><pre>")

		while data[n][0..8]==val
			fout.puts(data[n][4..(data[n].length-1)].sub(/Sun|Mon|Tue|Wed|Thu|Fri|Sat/, "")) if data[n] =~ res
			n = n + 1
			break if n==data.length
		end
		break if n==data.length
	}
	
	fout.puts("</pre></html></body>") if ext==".html"
	fout.close
}
end